class Api():
    '''
    Tokyo API properties class

    Attributes
    ----

    endpoints: dict
        `Returns all API endpoints`
    base: str
        `Returns API base URL`
    discord_server: str
        `Returns API discord server's URL`
    json_count: int
        `Returns API json endpoints count`
    buffer_count: int
        `Returns API buffer endpoints count`
    total_count: int
        `Returns API total endpoints count`
    '''

    def __init__(self) -> None:
        pass

    @property
    def endpoints(self) -> dict:
        '''Returns all API endpoints.'''
        return {
            'JSON': [
                '8ball',
                'ascii',
                'binary',
                'coinflip',
                'define',
                'discorduser',
                'gis',
                'github',
                'hastebin',
                'minecraft',
                'npm',
                'owoify',
                'repository',
                'reverse',
                'time',
                'topgg',
                'translate',
                'weather'
            ],
            'ANIME': [
                'gifs',
                'search',
            ],
            'BUFFER': [
                'bart',
                'beautiful',
                'blur',
                'boost',
                'circle',
                'clash',
                'color',
                'communism',
                'deepfry',
                'delete',
                'discordjs',
                'eject',
                'gay',
                'gray',
                'hollywood',
                'invert',
                'mad',
                'magik',
                'mexico',
                'overlay',
                'pixel',
                'rankcard',
                'rip',
                'santa',
                'ship',
                'shit',
                'simp',
                'sonic',
                'spotify',
                'supreme',
                'sus',
                'tweet',
                'walletcard',
                'wanted',
                'welcomecard',
                'whoreallyare'
            ]
        }

    @property
    def base(self) -> str:
        '''Returns API base URL'''
        return 'https://api.miduwu.ga'

    @property
    def discord_server(self) -> str:
        '''Returns API discord server's URL'''
        return 'https://discord.com/invite/3pT2WHG9EG'

    @property
    def json_count(self) -> int:
        '''Returns API JSON endpoints count'''
        return 18

    @property
    def buffer_count(self) -> int:
        '''Returns API buffer endpoints count'''
        return 36

    @property
    def total_count(self) -> int:
        '''Returns API total endpoints count'''
        return Api().json_count + Api().buffer_count


from .endpoints.json import Json
from .endpoints.buffer import Buffer
from .endpoints.anime import Anime

__title__ = 'tokyo'
__author__ = 'Loxitoh'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-present Loxitoh'
__version__ = '0.0.1'



import aiohttp
from io import BytesIO
from ..properties import Api

class Buffer:
    '''
    Tokyo API buffer class

    Methods
    ----

    get: ClientResponse
        `Get the bytes response of the given endpoint`
    '''

    async def get(endpoint: str = None, builders: dict = None) -> aiohttp.ClientResponse:
        '''Get the bytes response of the given endpoint'''

        dictionary = Api().endpoints

        if endpoint is None:
            raise TypeError('Parameters cannot be empty.')
        elif builders is None:
            raise TypeError('Parameters cannot be empty.')
        elif not isinstance(builders, dict):
            raise TypeError('Innapropiate argument type (dict is required).')
        elif endpoint not in dictionary['BUFFER']:
            raise TypeError(f'Could not find endpoint "{endpoint}" in the buffer endpoints dictionary.')

        async with aiohttp.ClientSession() as session:
            async with session.get(url = f'{Api().base}/image/{endpoint}', params = builders) as response:
                return BytesIO(await response.read()).getvalue()



import aiohttp
from ..properties import Api

class Anime:
    '''
    Tokyo API anime class

    Methods
    ----

    get: ClientResponse
        `Get the JSON response of the given endpoint`
    '''

    async def get(endpoint: str = None, builders: dict = None) -> aiohttp.ClientResponse:
        '''Get the JSON response of the given endpoint'''

        dictionary = Api().endpoints
        allowed_gif_types = [
            'angry',
            'baka',
            'bite',
            'blush',
            'cry',
            'dance',
            'deredere',
            'happy',
            'hug',
            'kiss',
            'path',
            'punch',
            'slap',
            'sleep',
            'smug'
        ]

        if endpoint is None:
            raise TypeError('Parameters cannot be empty.')
        elif builders is None:
            raise TypeError('Parameters cannot be empty.')
        elif not endpoint == 'gifs' and not isinstance(builders, dict):
            raise TypeError('Innapropiate argument type (dict is required).')
        elif endpoint == 'gifs' and not isinstance(builders, list):
            raise TypeError('Innapropiate argument type (list is required).')
        elif endpoint not in dictionary['ANIME']:
            raise TypeError(f'Could not find endpoint "{endpoint}" in the anime endpoints dictionary.')
        elif endpoint == 'gifs' and builders[0] not in allowed_gif_types:
            raise TypeError(f'Could not find gif type "{builders[0]}" in the anime gif types list.')
        elif endpoint == 'gifs' and builders[0] in allowed_gif_types:
            async with aiohttp.ClientSession() as session:
                async with session.get(url = f'{Api().base}/anime/{endpoint}/{builders[0]}') as response:
                    return await response.json()
        elif endpoint == 'search':
            async with aiohttp.ClientSession() as session:
                async with session.get(url = f'{Api().base}/anime/{endpoint}', params = builders) as response:
                    return await response.json()



from setuptools import setup
import re

requirements = []

with open('requirements.txt', 'r') as f:
    requirements = f.read().splitlines()

with open('README.md', 'r') as f:
    readme = f.read()

version = ''

with open('tokyo/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Could not find __version__ in __init__.py file.')

setup(
    name = 'tokyo.py',
    version = version,
    description = 'Python library for interacting with Tokyo API (https://api.miduwu.ga/)',
    long_description = readme,
    long_description_content_type = 'text/x-rst',
    url = 'https://github.com/Loxitoh/tokyo.py',
    author = 'Loxitoh',
    author_email = 'loxitoh@gmail.com',
    license = 'MIT',
    packages = [
        'tokyo',
        'tokyo.endpoints'
    ],
    include_package_data = True,
    install_requires = requirements
)
